
#ifndef bzmapper_h
#define bzmapper_h


#define BZMAPPER_NAME       "BZMAPPER"
#define BZMAPPER_VERSION    "G2.3"
#define BZMAPPER_VERSION_FLOAT  (2.3)   // run-time constant


#define MERGE_CONTAINED         // used in the merge_boxes()
// #define SAME_POLARITY_HOLES // used in generate_boxes()


/********************
 * Type Definitions *
 ********************/

typedef double vec3[3];


/************************
 * Supplied by bzmapper *
 ************************/

extern int load_map (const char *filename); // returns -1 on error

extern int stats_boxes (void);
extern int stats_pyramids (void);
extern int stats_tetras (void);
extern int stats_bases (void);
extern int stats_teleporters (void);
extern int stats_links (void);
extern int stats_weapons (void);
extern int stats_zones (void);

extern void set_ignore_names (int value);
extern void set_overlaps (void);
extern void set_merge_boxes (void);
extern void set_convert_old (void);

#define BZMAPPER_NEGSIZE    (1) // default for input
#define BZMAPPER_FLIPZ      (0) // default for output
extern void set_input_size_style (int value);
extern void set_output_size_style (int value);

#define BZMAPPER_ASCENDING  (+1)    // default
#define BZMAPPER_UNORDERED  (0)
#define BZMAPPER_DESCENDING (-1)
extern void set_z_ordering (int order);

extern void set_timeout (int secs); // set to 0 to disable
extern void set_runcount (int max); // set to 0 to disable

extern void set_debug (int debug);  // 0 to 9
extern void set_debug_objects (void);
extern void set_debug_teleporters (void);


/**********************************
 * Supplied by the client program *
 **********************************/

// These should return -1 on error

extern int make_world (double size, double flag_height);
extern int make_box (const char *name, vec3 pos, vec3 size, double rot,
                     int drivethru, int shootthru);
extern int make_pyr (const char *name, vec3 pos, vec3 size, double rot,
                     int drivethru, int shootthru, int flipz);
extern int make_tetra (const char *name, const vec3 *vertices,
                       unsigned char visible, unsigned char colored,
                       const unsigned char (*colors)[4],
                       int drivethru, int shootthru);
extern int make_tele (const char *name, vec3 pos, vec3 size,
                      double rot, double border, int drivethru,
                      int shootthru);
extern int make_base (const char *name, vec3 pos, vec3 size,
                      double rot, int color, int drivethru, int shootthru);
extern int make_zone (const char *name, vec3 pos, vec3 size, double rot,
                      const char *flags, int teams, int safety);
extern int make_weapon (const char *name, vec3 pos, vec3 size,
                        double rot, int drivethru, int shootthru,
                        double initdelay, double *delays, int delay_count,
                        const char *type);
extern int make_link (const char *name, int from, int to);
extern int make_options (const char *options);



#endif                          // bzmapper_h
