
/*
 *  E S C A N . H
 *
 *  Author:     Dave R (trepan)
 *  Date:       Feb 2003
 *
 *  A mathematical expression parser for the bzmapper program.
 *
 */

#ifndef escan_h
#define escan_h


#define ESCAN_MAX_FUNC_PARAMS		(32)


typedef union {
  double number;
  const char *string;
} parameter;

typedef double (*parsefunc) (parameter * params);



/*
 * escan()
 *
 * This is a handly little routine. It processes a string
 * according to the following formats:
 * 
 *   i : integer              (int *)     
 *   f : floating point       (double *)
 *   s : a string             (char *)     {no spaces}
 *   q : a quoted string      (char *)     {doesn't require ""'s}
 *   Q : a quoted string      (char *)     {requires ""'s}
 *   n : the last position    (char **)
 *
 *   w : only white space after the last conversion
 *       (if any other character is found, the function returns -1,
 *        not included in the returned count)
 *   b : expressions finish at a closing bracket, no space required
 *       (not included in the returned count)
 *   e : quotation are allowed to have octal and hex escape codes
 *       (not included in the returned count)
 *
 * No '%' character is required for a conversion.
 *
 */

extern int escan (const char *str, const char *format, ...);


extern int evaluate_string (const char *c, double *value);

extern int set_variable (const char *name, double value);
extern int set_constant (const char *name, double value);

extern void sprintf_double (char *str, double value);


/*
 * Functions can have different types of parameters
 * (currently only doubles and strings). To define a
 * function, you have to specify the type of each
 * parameter. Use 'f' for numerical values, and 's'
 * for string parameters. (ex: "ffsf");
 */
#define PARAM_TYPE_DOUBLE			'f'
#define PARAM_TYPE_STRING			's'

extern int add_function (const char *name, const char *argtypes,
                         parsefunc func);


extern void free_variables (void);
extern void free_functions (void);

extern void print_variables (void);
extern void print_functions (void);


#endif                          /* escan_h */
