
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "keywords.h"
#include "utils.h"


typedef struct {
  const char *name;
  int key;
} keyword_def;


/*
 * String Constants
 *
 * These do not have to line-up with the keys in 
 * [keyword.h]. The array has to be the same size,
 * but a level of indirect really is nice.
 *
 * Order these words for speed. It makes a difference.
 */

static const keyword_def Keywords[keywords_count] = {

  {"end", end_key},             /* definitely first up to bat */

  {"var", var_key},
  {"if", if_key},
  {"else", else_key},
  {"while", while_key},
  {"repeat", repeat_key},

  {"pos", position_key},
  {"rot", rotation_key},
  {"size", size_key},
  {"vertex", vertex_key},

  {"box", box_key},
  {"pyr", pyr_key},
  {"tetra", tetra_key},
  {"tele", tele_key},
  {"link", link_key},
  {"base", base_key},
  {"zone", zone_key},
  {"arc", arc_key},
  {"text", text_key},
  {"paths", paths_key},
  {"steps", steps_key},
  {"group", group_key},
  {"define", defgroup_key},

  {"drth", drivethrough_key},
  {"shth", shootthrough_key},
  {"passable", passable_key},
  {"visible", visible_key},
  {"flipz", flipz_key},

  {"name", name_key},
  {"modify", modify_key},
  {"delete", delete_key},
  {"undelete", undelete_key},
  {"intersect", intersect_key},
  {"hole", hole_key},
  {"tophole", tophole_key},
  {"point", point_key},
  {"angle", angle_key},
  {"to", to_key},
  {"from", from_key},
  {"tilt", tilt_key},
  {"color", color_key},
  {"vcolor", vcolor_key},
  {"border", border_key},
  {"segments", segments_key},
  {"closedloop", closedloop_key},
  {"convert", convert_key},

  {"flag", flag_key},
  {"team", team_key},
  {"safety", safety_key},

  {"pyramid", pyr_key},
  {"teleporter", tele_key},
  {"position", position_key},
  {"rotation", rotation_key},
  {"variable", var_key},
  {"const", const_key},
  {"constant", const_key},

  {"weapon", weapon_key},
  {"type", weapon_type_key},
  {"delay", weapon_delay_key},
  {"initdelay", weapon_init_delay_key},

  {"options", options_key},
  {"image", image_key},
  {"loadfont", loadfont_key},
  {"include", include_key},
  {"drivethrough", drivethrough_key},
  {"shootthrough", shootthrough_key},
  {"print", print_key},
  {"seed", seed_key},
  {"world", world_key},
  {"flagHeight", flag_height_key},
  {"quit", quit_key},

  {"flipzstyle", flipzstyle_key},
};


/******************************************************************************/
/******************************************************************************/

extern int
keyword_parse (const char *buf, int buflen, const char **pos)
{
  int i;
  int len;

  while ((*buf != '\0') && isspace (*buf)) {
    buf++;
  }

  for (i = 0; i < keywords_count; i++) {

    len = strlen (Keywords[i].name);

    if (len > buflen)
      return -1;

    if ((strncasecmp (Keywords[i].name, buf, (unsigned int) len) == 0) &&
        ((isspace (buf[len])) || (buf[len] == '\0'))) {
      *pos = buf + len;
      return Keywords[i].key;
    }
  }

  error ("unknown keyword: %s\n", buf);

  return -1;
}

/******************************************************************************/

extern const char *
keyword_string (int key)
{
  int i;

  for (i = 0; i < keywords_count; i++) {
    if (Keywords[i].key == key)
      return Keywords[i].name;
  }

  return "Unknown token!";
}

/******************************************************************************/

extern int
keyword_ended (int key)
{
  switch (key) {
    case group_key:
    case box_key:
    case pyr_key:
    case tetra_key:
    case base_key:
    case tele_key:
    case link_key:
    case zone_key:
    case weapon_key:
    case arc_key:
    case text_key:
    case image_key:
    case paths_key:
    case steps_key:
    case repeat_key:
    case modify_key:
    case hole_key:
    case tophole_key:
    case defgroup_key:
    case world_key:
    case options_key:
    case while_key:
    case if_key:
      return 1;
    default:
      return 0;
  }
}

/******************************************************************************/

extern int
keyword_top_level (int key)
{
  switch (key) {
    case box_key:
    case pyr_key:
    case tetra_key:
    case base_key:
    case tele_key:
    case link_key:
    case zone_key:
    case weapon_key:
    case world_key:

    case arc_key:
    case text_key:
    case image_key:
    case paths_key:
    case steps_key:
    case hole_key:
    case tophole_key:
    case group_key:
    case defgroup_key:

    case var_key:
    case const_key:
    case seed_key:
    case flipzstyle_key:
    case loadfont_key:
    case modify_key:
    case delete_key:
    case undelete_key:
    case intersect_key:
    case options_key:
    case print_key:
    case quit_key:
      return 1;
    default:
      return 0;
  }
}

/******************************************************************************/

extern int
keyword_flow_control (int key)
{
  switch (key) {
    case if_key:
    case else_key:
    case while_key:
    case repeat_key:
    case include_key:
    case var_key:
    case const_key:
    case print_key:
      return 1;
    default:
      return 0;
  }
}

/******************************************************************************/

int
keyword_is_end (const char *buf)
{
  while ((*buf != '\0') && isspace (*buf)) {
    buf++;
  }

  if ((strncasecmp ("end", buf, 3) == 0) &&
      (isspace (buf[3]) || (buf[3] == '\0'))) {
    return 1;
  }
  else {
    return 0;
  }
}

/******************************************************************************/

int
keyword_raw_input (int key)
{
  switch (key) {
    case options_key:
      return 1;
    default:
      return 0;
  }
}

/******************************************************************************/
/******************************************************************************/
