
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "list.h"


/****************************************************************************/

void
list_init (List * l)
{
  l->count = 0;
  l->items = NULL;
  l->chunk = LIST_CHUNK_DEFAULT;
}

/****************************************************************************/

int
list_add (void *item, List * l)
{
  void **tmparray;

  if ((l->count % l->chunk) == 0) {
    /*
     * Resize the array of pointers 
     */
    tmparray = (void **) realloc (l->items, sizeof (void *) *
                                  l->chunk * ((l->count / l->chunk) + 1));
    if (tmparray == NULL) {
      return -1;
    }
    else {
      l->items = tmparray;
    }
  }

  l->items[l->count] = item;
  l->count++;

  return 0;
}

/****************************************************************************/

int
list_find (void *item, List * l)
{
  int i;

  for (i = 0; i < l->count; i++) {
    if (l->items[i] == item)
      return i;
  }

  return -1;
}

/****************************************************************************/

int
list_rem_num (int item_num, List * l)
{
  void **tmparray;

  if ((item_num < 0) || (item_num > (l->count - 1)))
    return -1;

  if ((l->count % l->chunk) == 0) {
    /*
     * Resize the array of pointers 
     */
    tmparray = (void **) realloc (l->items, sizeof (void *) *
                                  l->chunk * (l->count / l->chunk));
    if ((tmparray == NULL) && ((l->count - 1) != 0)) {
      return -1;
    }
    else {
      l->items = tmparray;
    }
  }

  /*
   * Shift em back 
   */
  memmove (&l->items[item_num], &l->items[item_num + 1],
           (l->count - item_num - 1) * sizeof (void *));
  l->count--;

  return 0;
}

/****************************************************************************/

int
list_rem (void *item, List * l)
{
  int i;

  i = list_find (item, l);
  if (i == -1)
    return -1;

  return list_rem_num (i, l);
}

/****************************************************************************/

void
list_free (List * l)
{
  free (l->items);
  l->items = NULL;
  l->count = 0;

  return;
}

/****************************************************************************/

void
list_nuke (void (*free_func) (void *), List * l)
{
  int i;

  if (free_func != NULL) {
    for (i = 0; i < l->count; i++) {
      free_func (l->items[i]);
    }
  }
  else {
    for (i = 0; i < l->count; i++) {
      free (l->items[i]);
    }
  }

  free (l->items);
  l->items = NULL;
  l->count = 0;

  return;
}

/****************************************************************************/

void *
list_pop (List * l)
{
  void *item;

  if (l->count <= 0)
    return NULL;

  item = l->items[l->count - 1];
  if (list_rem_num (l->count - 1, l) == -1)
    return NULL;

  return item;
}

/****************************************************************************/

int
list_run (int (*run_func) (void *), List * l)
{
  int i;

  for (i = 0; i < l->count; i++) {
    if (run_func (l->items[i]) < 0)
      return i;
  }

  return -1;
}

/****************************************************************************/
/****************************************************************************/
