
#ifndef bzmutils_h
#define bzmutils_h

#include "bzmapper.h"


/*
 * For a consistent rand() function
 */
extern int my_rand (void);
extern void my_srand (unsigned int seed);
#define MY_RAND_MAX (32767)


/*
 * Floating point number comparison can be weird
 */
#define ACCURACY (0.000001)

#define IS_ZERO(x)              (fabs (x) < ACCURACY)
#define IS_EQUAL(x,y)           (fabs ((x) - (y)) < ACCURACY)



#define INDENT "  "
#define INDENT2 INDENT INDENT
#define INDENT3 INDENT2 INDENT
#define INDENT4 INDENT3 INDENT

#define LINE_LENGTH		(512)   /* excessive */


/*
 * Filename and Paths
 */

#ifndef _WIN32
#  define get_cwd(x,y) getcwd(x,y)
#  define PATH_SEPARATOR  "/"
#else
#  define get_cwd(x,y) _getcwd(x,y)
#  define PATH_SEPARATOR  "\\"
#  define strcasecmp stricmp
#  define strncasecmp strnicmp
#endif


extern int get_absolute_name (const char *current,
                              const char *target, char **absolute);
extern int get_relative_name (const char *current,
                              const char *target, char **relative);


/*
 * Warnings and Errors 
 */

extern void error (const char *format, ...);
extern void warning (const char *format, ...);


/*
 * For debugging 
 */

extern void clear_error (void);

extern int get_debug (void);
extern int get_debug_objects (void);
extern int get_debug_teleporters (void);

#define DEBUG_MASK      (0x0F)

#define DEBUG_OBJS_BIT  (1 << 4)
#define DEBUG_TELES_BIT (1 << 5)

#define DEBUG1	if ((get_debug() & DEBUG_MASK) >= 1) printf
#define DEBUG2	if ((get_debug() & DEBUG_MASK) >= 2) printf
#define DEBUG3	if ((get_debug() & DEBUG_MASK) >= 3) printf
#define DEBUG4	if ((get_debug() & DEBUG_MASK) >= 4) printf
#define DEBUG5	if ((get_debug() & DEBUG_MASK) >= 5) printf
#define DEBUG6	if ((get_debug() & DEBUG_MASK) >= 6) printf
#define DEBUG7	if ((get_debug() & DEBUG_MASK) >= 7) printf
#define DEBUG8	if ((get_debug() & DEBUG_MASK) >= 8) printf
#define DEBUG9	if ((get_debug() & DEBUG_MASK) >= 9) printf


#ifndef _WIN32
#  ifdef __APPLE__
#    include <machine/endian.h>
#  else
#    include <endian.h>
#  endif
#  include <stdint.h>
typedef int8_t s8;
typedef int16_t s16;
typedef int32_t s32;
typedef uint8_t u8;
typedef uint16_t u16;
typedef uint32_t u32;
#else
#  ifndef LITTLE_ENDIAN
#    define LITTLE_ENDIAN 4321
#  endif
#  define BYTE_ORDER LITTLE_ENDIAN
typedef signed char s8;
typedef unsigned char u8;
typedef signed short s16;
typedef unsigned short u16;
typedef signed int s32;
typedef unsigned int u32;
#endif


#endif                          /* bzmutils_h */
