
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "../boxfont.h"


#define GLYPHS_NAME			"DefGlyphs"
#define RECTS_NAME_BASE		"rects_"


static FILE *Outfile = NULL;


int
get_debug (void)
{
  return 0;
}

void
error (const char *format, ...)
{
  va_list arglist;

  va_start (arglist, format);
  vprintf (format, arglist);
  va_end (arglist);

  return;
}

static void
dump_glyph (int num, glyph_t * g)
{
  int i;

  if (g->rectcount != 0) {
    fprintf (Outfile, "static rect_t " RECTS_NAME_BASE
             "%03X[%i] = {\n", num, g->rectcount);

    for (i = 0; i < g->rectcount; i++) {
      fprintf (Outfile, "{%i, %i, %i, %i},\n", g->rects[i].x,
               g->rects[i].y, g->rects[i].width, g->rects[i].height);
    }

    fprintf (Outfile, "};\n");
  }

  return;
}

int
main (int argc, char **argv)
{
  int i;
  int count;
  glyph_t glyph;

  if (argc != 3) {
    printf ("usage:  makefont <input.psf> <output.h>\n");
    exit (1);
  }

  Outfile = fopen (argv[2], "w");
  if (Outfile == NULL) {
    perror ("fopen(Outfile)");
    exit (1);
  }

  if (init_boxfont (argv[1]) != 0) {
    printf ("init_boxfont(%s) failed\n", argv[1]);
    return -1;
  }


  if ((boxfont_char (255, &glyph) == 0) &&
      (boxfont_char (511, &glyph) == 0))
    count = 512;
  else
    count = 256;

  fprintf (Outfile, "/*\n");
  fprintf (Outfile, " * D E F F O N T . H\n");
  fprintf (Outfile, " *\n");
  fprintf (Outfile, " * Generated by utils/psf2rects.\n");
  fprintf (Outfile, " */\n");
  fprintf (Outfile, "\n");
  fprintf (Outfile, "#ifndef deffont_h\n");
  fprintf (Outfile, "#define deffont_h\n");
  fprintf (Outfile, "\n");
  fprintf (Outfile, "#include \"boxfont.h\"\n");
  fprintf (Outfile, "\n");
  fprintf (Outfile, "#define DEF_GLYPH_COUNT (%i)\n", count);
  fprintf (Outfile, "\n");

  for (i = 0; i < count; i++) {
    if (boxfont_char (i, &glyph) != 0) {
      printf ("boxfont_char(%i) failed.\n", i);
      exit (1);
    }
    else {
      dump_glyph (i, &glyph);
    }
  }

  fprintf (Outfile, "\n");
  fprintf (Outfile, "static glyph_t " GLYPHS_NAME "[%i] = {\n", count);

  for (i = 0; i < count; i++) {
    if (boxfont_char (i, &glyph) != 0) {
      printf ("boxfont_char(%i) failed.\n", i);
      exit (1);
    }

    if (glyph.rectcount != 0)
      fprintf (Outfile, "{%i, "
               RECTS_NAME_BASE "%03X},\n", glyph.rectcount, i);
    else
      fprintf (Outfile, "{0, NULL},\n");
  }
  fprintf (Outfile, "};\n");

  fprintf (Outfile, "#endif  /* deffont_h */\n");


  return 0;
}
